;Note: CGun object contains a current Ammo value and a MaxAmmo value.
;MaxAmmo=DWORD [obj+68h] ; Ammo=DWORD [obj+6Ch]
;Note2: object that contains two active CGun objects stores them at
;offsets 0E34h and 0E38h from the start of the object (for the currently
;held and currently holstered weapons, respectively)

;UPDATE (07/05/2006): Offset 0E38h for secondary weapon is only valid
;when the weapon is small, i.e., holstered to Anne's waist. If the weapon
;is larger, its instanced will be stored at 0E3Ch. Both must be verified
;when accessing a possible secondary weapon.

;object:
;DWORD [object] = VTable Pointer
;DWORD [object+4] = n/a
;DWORD [object+8] = Ammo value (0 if unused)
;DWORD [object+12] = AltAmmoCount value (0 if unused)
;DWORD [object+16] = number of times reload is to be performed per object
;DWORD [object+20] = which weapons to apply to
;DWORD [object+24] = saved number of times reload value (for re-init)
;ATX 1.59:
;DWORD [object+28] = number of CGun instances that can receive injection (0 for all instances)
;DWORD [object+32]... = list of CGun instanceIDs (in AT103 instance)

struct CAT103
       lpVTable 		dd	?
       refCount 		dd	?
       Ammo			dd	?
       AltAmmoCount		dd	?
       ReloadsLeft		dd	?
       TargetWeapons		dd	?
       InitialTotalReloads	dd	?
       NumberOfAllowedGuns	dd	?
       GunInstanceIDsList	dd	?	;(table of)
ends

align 16
;==========================================================
AT103MAIN:
;==========================================================
push edi
push ebx
push esi

	cmp DWORD [ecx+16],0
	jz NoMoreReload_103

mov edi,ecx

xor ebx,ebx

mov eax,[ActiveGunRef]
mov edx,[eax]

mov eax,[edx]
mov ecx,[eax+08h]

mov edx,[ecx+GETWEAPON_PRIMARY]     ;edx = possible primary CGun object
mov al,1		;check for 'hand' gun
call Verify_Reload_103

mov edx,[ecx+GETWEAPON_SECONDARYWAIST]	   ;edx = possible secondary CGun object
mov al,2		;check for holstered gun (at waist)
call Verify_Reload_103
mov edx,[ecx+GETWEAPON_SECONDARYBACK]	   ;edx = possible secondary CGun object
mov al,2		;check for holstered gun (one Anne's back)
call Verify_Reload_103

cmp DWORD [edi+16],-1
    jz NoLoopLimit_103	    ;no reload limit?

sub [edi+16],ebx	;only decrease reload count if a reload happened

NoLoopLimit_103:

NoMoreReload_103:
pop esi
pop ebx
pop edi
ret

;================================
;this verifies which gun the object specifies the reload
;to apply to and applies if needed using extra values if used

Verify_Reload_103:	;al = test value to specified weapon
	test edx,edx
	mov esi,[CGUNVTABLE]
		jz NoWeapon_103   ;make sure Anne is holding an object
	cmp [edx],esi
	    jnz NoWeapon_103	  ;make sure we have a CGun type object

   ;if applicable, test if gun instance can receive ammo injection
	cmp DWORD [edi+28],0
	    jz Verify_Reload_103_ALLINSTANCES
	push eax
	push ecx
	push edx
	mov eax,[edx+CInstance.InstanceNameID]
	lea edx,[edi+32]
	mov ecx,[edi+28]
	Verify_Reload_103_VerifyInstanceLoop:
	cmp eax,[edx]
	    jz Verify_Reload_103_VerifyInstanceLoop_FOUND
	add edx,4
	dec ecx
	    jnz Verify_Reload_103_VerifyInstanceLoop
	pop edx
	pop ecx
	pop eax
	jmp NoWeapon_103	      ;not found
	Verify_Reload_103_VerifyInstanceLoop_FOUND:
	pop edx
	pop ecx
	pop eax
	Verify_Reload_103_ALLINSTANCES:

	test BYTE [edi+16],al
		jz NoWeapon_103

	mov eax,[edi+8] 	;Ammo value
	test eax,eax
		jz NoAmmoValue_103
	mov [edx+CGun.AmmoCount],eax
	or ebx,1
	jmp NoWeapon_103

NoAmmoValue_103:
	mov eax,[edx+CGun.MaxAmmoCount]       ;object's MaxAmmo value
	sub eax,[edi+12]	;substract AltAmmoCount value
	mov [edx+CGun.AmmoCount],eax
	or ebx,1

NoWeapon_103:
ret

align 16
;==========================================================
AT103_REINIT:
;==========================================================
mov eax,[ecx+24]
mov [ecx+16],eax
mov eax,[esp+4] 	;standard
ret 4

align 16
;====================================================================
_AT103Init:
;====================================================================
;check for possible instances
;+28 = number of guns to apply effect to (0 for all)
;+32 = pointer to gun instance IDs if count non-null

mov ebx,-1
mov esi,stack_ext_buffer

;get Axx instances until not found, starting with A00
_AT103Init_GetInstances_LOOP:

add ebx,1
lea edx,[A00+ebx*8]
GETVALUE edx,ebx
SETVALUE2 value_string
test eax,eax
     jz _AT103Init_GetInstances_LOOP_EXIT

invoke GETTEMPINST_PROC,eax,0,1    ;returns string ID (case-sensitive, unique)
add esp,12
test eax,eax
     jz _AT103Init_GetInstances_LOOP_EXIT

mov [esi],eax
add esi,4
cmp esi,(stack_ext_buffer+200*4)
    jb _AT103Init_GetInstances_LOOP
_AT103Init_GetInstances_LOOP_EXIT:

sub esi,stack_ext_buffer
lea edx,[esi+32]
shr esi,2				 ;esi = number of potential gun instances

;create main AT103 instance
ALLOCAT edx				 ;size = 32 + 4*number of gun instances         
lea eax,[AT103]
mov [edi],eax
    xor eax,eax
mov [edi+4],eax
mov [edi+8],eax
mov [edi+12],eax
    inc eax
mov [edi+16],DWORD -1
mov [edi+20],eax
mov [edi+28],esi			;instance count (0 if apply to all)

;copy instances to AT103 instance (if any)
test esi,esi
     jz _AT103Init_NoInstancesA
push edi
mov ecx,esi
mov esi,stack_ext_buffer
add edi,32
rep movsd
pop edi
_AT103Init_NoInstancesA:

;get other reload values
inc ebx
GETVALUE Ammo,ebx
SETVALUE value_int,3

inc ebx
GETVALUE AltAmmoCount,ebx
SETVALUE value_int,4

inc ebx
GETVALUE LoopCount,ebx
SETVALUE value_int,5

inc ebx
GETVALUE Frame,ebx
SETVALUE value_int,6

mov eax,[edi+16]
mov [edi+24],eax

	jmp ExitINAT