;============================================================
;ACTIONTYPE 114 INITIALIZATION CODE
;============================================================

;fetch group listing for A00 main group (holds instance names to teleport)
;note: does not keep track of counters for main directory (which holds
; int ActionType = 114 entry). Only keeps track of counter for one
; sub-group at a time (more than one not necessary due to groups being
; scanned only one at a time).

;note: In this code, LOCATION GROUP refers to the SUB-GROUPs in
;the documentation, while SUB-GROUPS refer to the LOCATION groups
;in the documentation (interchanged for the sake of confusion)

      xor edi,edi	    ;edi = official AT114 instance
      xor ebx,ebx	    ;ebx = buffer for group pointers
      mov [AT114_MAINGROUPCounter],ebx
      mov [AT114_TotalProbabilityCount],ebx
      mov [AT114_ExtraAllocBytes],ebx
      mov [AT114_TotalAllocations],ebx
      mov [AT114_TotalAllocationsSize],ebx

;switch to MAIN GROUP (A00)
      mov ecx,AT114_SUBGROUPCounter1
      mov edx,AT114_SUBGROUPIDValue1
      mov eax,[ebp-28]
      mov DWORD [ecx],0
      add eax,A00	      ;table displacement for the T-Script value
      invoke GetGroupIDValueProc,edx,eax,[ebp-32],[ebp-36],ecx
      add esp,20	 ;returns al=false if not found/error
      mov edx,[AT114_SUBGROUPIDValue1]
      test al,al
      mov [AT114_LastError],AT114Error_NoMAINGROUP
	   jz AT114_ExitInit_NoInitialValues
      test edx,edx
	   jz AT114_ExitInit_NoInitialValues	    ;group value exists?

;get int Frame value = number of groups
      mov eax,[ebp-28]
      add eax,Frame
      invoke GetEntryValueProc,eax,edx,[ebp-36],[AT114_SUBGROUPCounter1]    ;first entry in group
      add esp,16
      test eax,eax
      mov [AT114_LastError],AT114Error_InvalidFrameValue
      mov [AT114_LastError_DWORDParam1],0
	   jz AT114_ExitInit_NoInitialValues
      cmp DWORD [eax+4],value_int
	  jnz AT114_ExitInit_NoInitialValues
      mov edx,[eax+28h] 	;value of Frame
      cmp edx,0
      mov [AT114_LastError_DWORDParam1],edx
	  jle AT114_ExitInit_NoInitialValues
      cmp edx,200		;number of GROUP (Axx) values should be between 1 and 199 inclusively
	  jnb AT114_ExitInit_NoInitialValues

      mov [AT114_NumberOfGroups],edx
      inc [AT114_SUBGROUPCounter1]

;get int Sample value = number of Axx instances (don't want to scan all for nothing)
      mov edx,[AT114_SUBGROUPIDValue1]
      mov eax,[ebp-28]
      add eax,Sample
      invoke GetEntryValueProc,eax,edx,[ebp-36],[AT114_SUBGROUPCounter1]
      add esp,16
      test eax,eax
      mov [AT114_LastError],AT114Error_InvalidSampleValue
      mov [AT114_LastError_DWORDParam1],0
	   jz AT114_ExitInit_NoInitialValues
      cmp DWORD [eax+4],value_int
	  jnz AT114_ExitInit_NoInitialValues
      mov edx,[eax+28h] 	;value of Sample
      cmp edx,0
      mov [AT114_LastError_DWORDParam1],0
	  jle AT114_ExitInit_NoInitialValues
      cmp edx,200		;number of Axx values should be between 1 and 199 inclusively
	  ja AT114_ExitInit_NoInitialValues

      mov [AT114_NumberOfInstances],edx
      inc [AT114_SUBGROUPCounter1]

;fetch instances to teleport

      mov esi,1
      AT114_GetInstancesLoop:
      mov edx,[AT114_SUBGROUPIDValue1]
      mov eax,[ebp-28]
      lea eax,[eax+8*esi+A00]
      invoke GetEntryValueProc,eax,edx,[ebp-36],[AT114_SUBGROUPCounter1]
      add esp,16
      test eax,eax
      mov [AT114_LastError],AT114Error_MissingInstance
      mov [AT114_LastError_DWORDParam1],esi
	  jz AT114_ExitInit_NoInitialValues
      cmp DWORD [eax+4],value_string
	  jnz AT114_ExitInit_NoInitialValues
      inc [AT114_SUBGROUPCounter1]
      mov eax,[eax+40h]
      test eax,eax
	   jz AT114_ExitInit_NoInitialValues
      invoke GETTEMPINST_PROC,eax,0,1
      add esp,12
      test eax,eax
	   jz AT114_ExitInit_NoInitialValues
      mov [AT114_InstanceIDBuffer+4*esi-4],eax
      inc esi
      cmp esi,[AT114_NumberOfInstances]
	  jbe AT114_GetInstancesLoop

;create main temporary buffer, holds pointers for all groups (to location info structures)
;     ebx = temporary buffer
;     size = 5*(number of groups)*(number of groups)
;     (first part = series of BYTE flags, second half = series of DWORD pointers)

      xor ebx,ebx
      mov eax,[AT114_NumberOfGroups]
      mov ecx,[AT114_NumberOfInstances]
      mul ecx
      mov [AT114_NumberOfGroupsXInstances],eax
      lea eax,[eax+eax*4]
      mov ecx,[AT114_TotalAllocationsSize]
      mov [AT114_TempBufferSize],eax
      mov [AT114_LastError_DWORDParam1],eax
      mov [AT114_LastError_DWORDParam2],AT114Error_NoLocalMem_maintemp
      mov [AT114_LastError_DWORDParam3],0
      add ecx,eax
      mov [AT114_TotalAllocationsSize],ecx
      mov [AT114_LastError_DWORDParam4],ecx
      invoke LocalAlloc,LMEM_FIXED,eax
      test eax,eax
      mov [AT114_LastError],AT114Error_NoLocalMem
	   jz AT114_ExitInit_NoInitialValues
      inc [AT114_TotalAllocations]
      mov ebx,eax

;set initial flags for all groups to none
      push edi
      mov ecx,[AT114_NumberOfGroupsXInstances]
      mov edi,ebx
      ZEROMEM				      ;default flags = 0

;set initial group probability values to zero
      mov ecx,[AT114_NumberOfGroups]
      xor eax,eax
      mov edi,AT114_GroupProbabilities
      rep stosd
      pop edi

;MAIN GROUPS FETCH LOOP
      mov [AT114_SUBGROUPCountCurrent],1
      mov esi,ebx
      mov [AT114_FlagsPointerCurrent],ebx	  ;this mem is storage for location entry flags
      add esi,[AT114_NumberOfGroupsXInstances]	  ;esi points to storage for location entry pointers

      AT114_GetGroupsMainLoop:

;get next SUB-GROUP 1
      mov edx,[AT114_SUBGROUPCountCurrent]
      mov ecx,AT114_SUBGROUPCounter1
      mov eax,[ebp-28]
      mov DWORD [ecx],0
      lea eax,[eax+edx*8+A00]
      invoke GetGroupIDValueProc,AT114_SUBGROUPIDValue1,eax,[ebp-32],[ebp-36],ecx
      add esp,20
      test al,al
      mov edx,[AT114_SUBGROUPIDValue1]
	   jz AT114_GetGroupsMainLoop_NoGroup
      test edx,edx
	   jz AT114_GetGroupsMainLoop_NoGroup

      ;get LoopCount probability value from this SUB-GROUP
	   mov eax,[ebp-28]
	   add eax,LoopCount
	   GETVALUEEXT eax,[AT114_SUBGROUPIDValue1],DWORD [ebp-36],\
		       AT114_SUBGROUPCounter1,value_int
	   test ecx,ecx 	   ;was entry valid
		jz AT114_GetGroupsMainLoop_NoGroup
	   cmp eax,0		   ;is value zero or negative?
	       jle AT114_GetGroupsMainLoop_NoGroup
	   add [AT114_TotalProbabilityCount],eax

      ;save probability count for this group
	   mov ecx,[AT114_SUBGROUPCountCurrent]
	   mov [AT114_GroupProbabilities+ecx*4-4],eax

      ;SUB-LOOP LOCATION VALUES FETCH FOR CURRENT GROUP
	   mov [AT114_LOCATIONGROUPCountCurrent],1

	   AT114_GetLocationsSubLoop:

      ;get next sub-group containing location values
	   mov edx,[AT114_LOCATIONGROUPCountCurrent]
	   mov ecx,AT114_SUBGROUPCounter2
	   mov eax,[ebp-28]
	   mov DWORD [ecx],0
	   lea eax,[eax+edx*8+A00]
	   invoke GetGroupIDValueProc,AT114_SUBGROUPIDValue2,eax,[AT114_SUBGROUPIDValue1],[ebp-36],ecx
	   add esp,20
	   test al,al
	   mov edx,[AT114_SUBGROUPIDValue2]
	       jz AT114_GetLocationsSubLoop_NoGroup
	   test edx,edx
		jz AT114_GetLocationsSubLoop_NoGroup

      ;set default flags for this location group (excludes enable flag)
	   mov edx,[AT114_FlagsPointerCurrent]
	   mov BYTE [edx],AT114_LOC_USEDESTOBJECT+AT114_LOC_USEONTERRAIN+AT114_LOC_USESETORIENTATION+\
			  AT114_LOC_USESETPOSITION+AT114_LOC_USEHEIGHTRELATIVE	      ;defaults

      ;check for SetOrientation bool
	   mov edx,[AT114_SUBGROUPIDValue2]
	   mov eax,[ebp-28]
	   add eax,SetOrientation
	   invoke GetEntryValueProc,eax,edx,[ebp-36],[AT114_SUBGROUPCounter2]
	   add esp,16
	   test eax,eax
		jz AT114_NoSetOrientationValueChange
	   cmp DWORD [eax+4],value_bool
	       jnz AT114_NoSetOrientationValueChange
	   inc [AT114_SUBGROUPCounter2]
	   test BYTE [eax+28h],1
		jnz AT114_NoSetOrientationValueChange
	   mov edx,[AT114_FlagsPointerCurrent]
	   and BYTE [edx],(not AT114_LOC_USESETORIENTATION)
	   AT114_NoSetOrientationValueChange:

      ;check for SetPosition bool
	   mov edx,[AT114_SUBGROUPIDValue2]
	   mov eax,[ebp-28]
	   add eax,SetPosition
	   invoke GetEntryValueProc,eax,edx,[ebp-36],[AT114_SUBGROUPCounter2]
	   add esp,16
	   test eax,eax
		jz AT114_NoSetPositionValueChange
	   cmp DWORD [eax+4],value_bool
	       jnz AT114_NoSetPositionValueChange
	   inc [AT114_SUBGROUPCounter2]
	   test BYTE [eax+28h],1
		jnz AT114_NoSetPositionValueChange
	   mov edx,[AT114_FlagsPointerCurrent]
	   and BYTE [edx],(not AT114_LOC_USESETPOSITION)
	   AT114_NoSetPositionValueChange:

      ;check if position and orientation are set
	   mov edx,[AT114_FlagsPointerCurrent]
	   test BYTE [edx],(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
		jz AT114_GetLocationsSubLoop_NoGroup		;group cannot have no location and no orientation set

      ;check for HeightRelative bool
	   mov edx,[AT114_SUBGROUPIDValue2]
	   mov eax,[ebp-28]
	   add eax,HeightRelative
	   invoke GetEntryValueProc,eax,edx,[ebp-36],[AT114_SUBGROUPCounter2]
	   add esp,16
	   test eax,eax
		jz AT114_NoHeightRelativeValueChange
	   cmp DWORD [eax+4],value_bool
	       jnz AT114_NoHeightRelativeValueChange
	   inc [AT114_SUBGROUPCounter2]
	   test BYTE [eax+28h],1
		jnz AT114_NoHeightRelativeValueChange
	   mov edx,[AT114_FlagsPointerCurrent]
	   and BYTE [edx],(not AT114_LOC_USEHEIGHTRELATIVE)
	   AT114_NoHeightRelativeValueChange:

      ;check for OnTerrain bool
	   mov edx,[AT114_SUBGROUPIDValue2]
	   mov eax,[ebp-28]
	   add eax,OnTerrain
	   invoke GetEntryValueProc,eax,edx,[ebp-36],[AT114_SUBGROUPCounter2]
	   add esp,16
	   test eax,eax
		jz AT114_NoOnTerrainValueChange
	   cmp DWORD [eax+4],value_bool
	       jnz AT114_NoOnTerrainValueChange
	   inc [AT114_SUBGROUPCounter2]
	   test BYTE [eax+28h],1
		jnz AT114_NoOnTerrainValueChange
	   mov edx,[AT114_FlagsPointerCurrent]
	   and BYTE [edx],(not AT114_LOC_USEONTERRAIN)
	   AT114_NoOnTerrainValueChange:

      ;check for TeleportDestObjectName
	   mov edx,[AT114_SUBGROUPIDValue2]
	   mov eax,[ebp-28]
	   add eax,TeleportDestObjectName
	   invoke GetEntryValueProc,eax,edx,[ebp-36],[AT114_SUBGROUPCounter2]
	   add esp,16
	   test eax,eax
		jz AT114_NoTeleportDestObjectName
	   cmp DWORD [eax+4],value_string
	       jnz AT114_NoTeleportDestObjectName
	   inc [AT114_SUBGROUPCounter2]
	   cmp DWORD [eax+28h],0
	       jle AT114_NoTeleportDestObjectName
	   mov eax,[eax+40h]
	   test eax,eax
		jz AT114_NoTeleportDestObjectName
	   invoke GETTEMPINST_PROC,eax,0,1
	   add esp,12
	   test eax,eax
		jz AT114_NoTeleportDestObjectName
	   mov [esi],eax		       ;store string ID
	   jmp AT114_GetLocationsSubLoop_GroupFound
	   AT114_NoTeleportDestObjectName:

      ;since not a Teleport Dest Object, use X,Y,Z values
	   mov edx,[AT114_FlagsPointerCurrent]
	   mov al,[edx]
	   and BYTE [edx],(not AT114_LOC_USEDESTOBJECT)
	   and al,(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
	   mov ecx,(3*4)
	   add [AT114_ExtraAllocBytes],ecx
	   cmp al,(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
	       jnz AT114_UseOnlyOneTripletValueSet
	       mov ecx,(6*4)		;(last member is not immediately used)
	       add [AT114_ExtraAllocBytes],(4*4)	 ;count extra DWORD for quaternion conversion later
	   AT114_UseOnlyOneTripletValueSet:

      ;allocate temporary memory for values

	   mov eax,[AT114_TotalAllocations]
	   mov edx,[AT114_TotalAllocationsSize]
	   mov [AT114_LastError_DWORDParam1],ecx
	   mov [AT114_LastError_DWORDParam2],AT114Error_NoLocalMem_smalltemp
	   mov [AT114_LastError_DWORDParam3],eax
	   add edx,ecx
	   mov [AT114_TotalAllocationsSize],edx
	   mov [AT114_LastError_DWORDParam4],edx
	   invoke LocalAlloc,LMEM_FIXED,ecx
	   test eax,eax
	   mov [AT114_LastError],AT114Error_NoLocalMem
		jz AT114_ExitInit
	   inc [AT114_TotalAllocations]
	   mov [esi],eax

      ;get X,Y,Z values, if applicable
	   mov ecx,eax

	   mov [AT114_LastError],AT114Error_InvalidXYZ
	   mov eax,[AT114_LOCATIONGROUPCountCurrent]
	   mov edx,[AT114_SUBGROUPCountCurrent]
	   mov [AT114_LastError_DWORDParam2],eax
	   mov [AT114_LastError_DWORDParam3],edx	  ;(error values)

	   mov edx,[AT114_FlagsPointerCurrent]
	   test BYTE [edx],AT114_LOC_USESETPOSITION
		jz AT114_NoFetchPositionValues
	   mov eax,[ebp-28]
	   add eax,X
	   stdcall GetTripletValues,eax,[AT114_SUBGROUPIDValue2],[ebp-36],\
		   AT114_SUBGROUPCounter2,value_float,ecx
	   test eax,eax
	   mov [AT114_LastError_DWORDParam1],AT114Error_InvalidXYZ_xyz
		jz AT114_ExitInit	;or AT114_GetLocationsSubLoop_NoGroup
	   mov ecx,[esi]
	   add ecx,12
	   AT114_NoFetchPositionValues:

      ;get XFree,YFree,ZFree values, if applicable (will be converted to quaternion later, during memcopy)
	   mov edx,[AT114_FlagsPointerCurrent]
	   test BYTE [edx],AT114_LOC_USESETORIENTATION
		jz AT114_GetLocationsSubLoop_GroupFound
	   mov eax,[ebp-28]
	   add eax,XFree
	   stdcall GetTripletValues,eax,[AT114_SUBGROUPIDValue2],[ebp-36],\
		   AT114_SUBGROUPCounter2,value_float,ecx
	   test eax,eax
	   mov [AT114_LastError_DWORDParam1],AT114Error_InvalidXYZ_xyzfree
		jz AT114_ExitInit	;or AT114_GetLocationsSubLoop_NoGroup

      ;set enable flag for this location group
	   AT114_GetLocationsSubLoop_GroupFound:
	   mov edx,[AT114_FlagsPointerCurrent]
	   or BYTE [edx],AT114_LOC_ENTRYVALID

      ;end of LOCATION GROUP loop
	   AT114_GetLocationsSubLoop_NoGroup:
	   mov eax,[AT114_LOCATIONGROUPCountCurrent]
	   add esi,4
	   inc eax
	   inc [AT114_FlagsPointerCurrent]
	   cmp eax,[AT114_NumberOfInstances]
	   mov [AT114_LOCATIONGROUPCountCurrent],eax
	       jbe AT114_GetLocationsSubLoop

;end of SUB GROUP loop
      AT114_GetGroupsMainLoop_NoGroup:
      mov eax,[AT114_SUBGROUPCountCurrent]
      inc eax
      cmp eax,[AT114_NumberOfGroups]
      mov [AT114_SUBGROUPCountCurrent],eax
	  jbe AT114_GetGroupsMainLoop

;make sure total probability count is not zero
      cmp [AT114_TotalProbabilityCount],0
      mov [AT114_LastError],AT114Error_InvalidProb
	  jz AT114_ExitInit

;allocate memory for main instance and organize
;structure:
;sizeof.AT114Instance
;4*(number of instances)                        = instance IDs
;4*(number of groups)                           = group probabilities list
;1*(number of instances)*(number of groups)     = BYTE flags for each Location entry (present or not)
;4*(number of instances)*(number of groups)     = series of pointers for each location entry
;                                                 (pointers are either to x,y,z locations in next section
;                                                  or holds pointer to some instance if DestObjectName used)
;(space reserved = size defined by [AT114_ExtraAllocBytes]) = holds x,y,z data pointed to for locations w/o dest.objects

      mov eax,[AT114_NumberOfInstances]
      mov ecx,[AT114_NumberOfGroups]
      shl eax,2
      shl ecx,2
      mov esi,sizeof.AT114Instance
      mov edx,[AT114_NumberOfGroupsXInstances]
      add esi,eax
      add esi,ecx
      lea eax,[edx*4]
      mov ecx,[AT114_ExtraAllocBytes]
      add esi,edx
      add esi,eax	   ;esi will point to spare space for X,Y,Z values
      add ecx,esi

;allocate official instance memory
      mov [AT114_InstanceSize],ecx

      mov eax,[AT114_TotalAllocations]
      mov edx,[AT114_TotalAllocationsSize]
      mov [AT114_LastError_DWORDParam1],ecx
      mov [AT114_LastError_DWORDParam2],AT114Error_NoLocalMem_mainobj
      mov [AT114_LastError_DWORDParam3],eax
      add edx,ecx
      mov [AT114_TotalAllocationsSize],edx
      mov [AT114_LastError_DWORDParam4],edx

      invoke YAPAXI,ecx
      add esp,4
      test eax,eax
      mov edi,eax
      mov [AT114_LastError],AT114Error_NoLocalMem
	  jz AT114_ExitInit
      inc [AT114_TotalAllocations]

      add esi,edi	     ;esi points to space for x,y,z data

      lea eax,[AT114]
      mov [edi+AT114Instance.lpVTable],eax
      mov [edi+AT114Instance.Reserved],0
      mov eax,[AT114_TotalProbabilityCount]
      mov ecx,[AT114_NumberOfGroups]
      mov edx,[AT114_NumberOfInstances]
      mov [edi+AT114Instance.TotalProbabilityCount],eax
      mov [edi+AT114Instance.NumberOfGroups],ecx
      mov [edi+AT114Instance.NumberOfInstances],edx

      lea ecx,[edi+sizeof.AT114Instance]
      mov eax,[AT114_NumberOfInstances]
      mov edx,[AT114_NumberOfGroups]
      shl eax,2
      shl edx,2
      mov [edi+AT114Instance.lpInstanceList],ecx
      add ecx,eax
      mov [edi+AT114Instance.lpGroupsProbabilitiesList],ecx
      add ecx,edx
      mov [edi+AT114Instance.lpGroupLocationFlags],ecx
      add ecx,[AT114_NumberOfGroupsXInstances]
      mov [edi+AT114Instance.lpGroupData],ecx

;copy instance ID and group probability buffers
	push esi
	push edi

	push [edi+AT114Instance.lpGroupsProbabilitiesList]
	mov edi,[edi+AT114Instance.lpInstanceList]
	mov esi,AT114_InstanceIDBuffer
	mov ecx,[AT114_NumberOfInstances]
	rep movsd

	pop edi
	mov esi,AT114_GroupProbabilities
	mov ecx,[AT114_NumberOfGroups]
	rep movsd

	pop edi
	pop esi

;copy location flags, set up location pointers, and copy potential x,y,z values
;while freeing temporary memory. Also, convert x,y,z values (if present) to quaternions.
;Implicitly keeps order.

       ;esi = storage for x,y,z values
       ;ebx = counter for location values
	mov [AT114_TempBufferBaseFlags],ebx
	add ebx,[AT114_NumberOfGroupsXInstances]
	mov [AT114_TempBufferBasePointers],ebx

	xor ebx,ebx

	AT114_CopyGroupDataLoop:

;copy flags
	mov edx,[AT114_TempBufferBaseFlags]
	mov ecx,[edi+AT114Instance.lpGroupLocationFlags]
	mov al,[ebx+edx]
	mov [ebx+ecx],al
	test al,AT114_LOC_ENTRYVALID
	     jz AT114_CopyGroupDataLoop_LeaveEntry
	test al,AT114_LOC_USEDESTOBJECT
	     jz AT114_CopyGroupDataLoop_UseXYZ

;if dest. object, simply copy instance ID
	mov edx,[AT114_TempBufferBasePointers]
	mov ecx,[edi+AT114Instance.lpGroupData]
	mov eax,[ebx*4+edx]
	mov [ebx*4+ecx],eax
	jmp AT114_CopyGroupDataLoop_LeaveEntry
	AT114_CopyGroupDataLoop_UseXYZ:

;if XYZ, check entry for orientation/position and copy accordingly
	mov DWORD [ebp-4],3
	and al,(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
	cmp al,(AT114_LOC_USESETORIENTATION+AT114_LOC_USESETPOSITION)
	mov BYTE [ebp-8],al
	    jnz AT114_CopyGroupDataLoop_NotBoth
	    mov DWORD [ebp-4],6
	AT114_CopyGroupDataLoop_NotBoth:

	mov edx,[AT114_TempBufferBasePointers]
	mov ecx,[edi+AT114Instance.lpGroupData]
	mov eax,[ebx*4+edx]		       ;get x,y,z values in separate memory location
	mov [ebp-12],eax
	mov [ebx*4+ecx],esi		       ;store pointer in AT114 instance to new space for x,y,z values

	push edi
	mov edi,esi
	mov esi,eax
	mov ecx,[ebp-4]
	rep movsd			       ;copy all x,y,z values for this location
	mov edx,esi
	mov esi,edi
	pop edi

	;if orientation values were used, convert the three last members
	;(three angles) to a four-member quaternion (increase size by 4)
	test BYTE [ebp-8],AT114_LOC_USESETORIENTATION
	     jz AT114_CopyGroupDataLoop_LeaveEntry_Free

	add esi,4
	lea ecx,[edx-12]
	lea edx,[esi-16]
	fld DWORD [ecx]
	fmul [pidiv180]
	fld DWORD [ecx+4]
	fmul [pidiv180]
	fld DWORD [ecx+8]
	fmul [pidiv180]
	fstp DWORD [ecx+8]
	fstp DWORD [ecx+4]
	fstp DWORD [ecx]
	call XYZDegreesToQuaternion

	AT114_CopyGroupDataLoop_LeaveEntry_Free:
	invoke LocalFree,[ebp-12]

	AT114_CopyGroupDataLoop_LeaveEntry:

	inc ebx
	cmp ebx,[AT114_NumberOfGroupsXInstances]
	    jb AT114_CopyGroupDataLoop

;finalize process
	invoke LocalFree,[AT114_TempBufferBaseFlags]
	call RANDOMIZE_RANDOM_PROC	;seed random value in advance
	mov [AT114_LastError],AT114Error_Success
	mov [AT114_LastError_DWORDParam1],edi
	mov eax,[AT114_InstanceSize]
	mov ecx,[AT114_NumberOfGroups]
	mov [AT114_LastError_DWORDParam2],eax
	mov [AT114_LastError_DWORDParam3],ecx
	mov eax,[AT114_NumberOfInstances]
	mov ecx,[AT114_TotalProbabilityCount]
	mov [AT114_LastError_DWORDParam4],eax
	mov [AT114_LastError_DWORDParam5],ecx
	jmp AT114_ExitInit_Done

AT114_ExitInit:
	mov eax,[AT114_NumberOfGroupsXInstances]
	mov esi,ebx
	mov [ebp-12],ebx
	push edi
	lea edi,[esi+eax]
	add ebx,[AT114_TempBufferSize]
	AT114_ExitInit_TempBufferFreeLoop:
	mov al,[esi]
	test al,AT114_LOC_ENTRYVALID
	     jz AT114_ExitInit_TempBufferFreeLoop_Skip
	test al,AT114_LOC_USEDESTOBJECT
	     jnz AT114_ExitInit_TempBufferFreeLoop_Skip
	invoke LocalFree,[edi]
	AT114_ExitInit_TempBufferFreeLoop_Skip:
	add edi,4
	add esi,1
	cmp edi,ebx
	    jb AT114_ExitInit_TempBufferFreeLoop
	invoke LocalFree,[ebp-12]
	pop edi

AT114_ExitInit_NoInitialValues:
AT114_ExitInit_Done:

;print last error message to ATX_LOGFILE.txt
	mov ebx,edi
	mov esi,AT114_Error_Main
	mov edi,AT114_MessageBuffer
	call TransferString
	mov [AT114_TempVar],esp
	invoke sprintf,edi,[AT114_LastError],[AT114_LastError_DWORDParam1],\
	       [AT114_LastError_DWORDParam2],[AT114_LastError_DWORDParam3],\
	       [AT114_LastError_DWORDParam4],[AT114_LastError_DWORDParam5]
	add eax,(AT114_Error_Main_end-AT114_Error_Main)
	mov esp,[AT114_TempVar]
	stdcall WriteLogFileInGame,AT114_MessageBuffer,eax
	mov edi,ebx

	cmp [AT114_LastError],AT114Error_Success
	    jnz UnableToAllocateMem
	jmp ExitINAT
